from typing import Dict, Optional, Type

from fck.cktype.cktypeinterface import CKTYPEINTERFACE as CKTYPEINTERFACE

from .crc32 import CRC32
from .md5 import MD5

CKTYPES: Dict[str, Type[MD5 | CRC32]] = {
    "MD5": (MD5),
    "CRC32": (CRC32),
}


def resolve(fname: str, esum: Optional[str] = None, cstype: str = ""):
    """
    Checks fname input for checksum patterns and returns first match.
    Can be overridden with cstype.
    If neither is applicable it defaults to CRC32.
    """
    cstype = cstype.upper()
    if cstype in CKTYPES:
        if esum is None and (match := CKTYPES[cstype].REGEX.search(fname)):
            esum = match.group(0)
        return CKTYPES[cstype](), esum

    for cstname, csclass in CKTYPES.items():
        if match := csclass.REGEX.search(fname):
            if esum is None:
                esum = match.group(0)
            return csclass(), esum

    return CRC32(), esum
